/** SocketManager */
package emporer.server;

import org.json.JSONArray;
import org.json.JSONObject;
import emporer.domain.User;
import emporer.domain.action.Action;
import emporer.domain.session.Request;
import emporer.domain.session.Response;
import emporer.domain.session.UserSession;
import emporer.domain.session.UserSessionThreadLocal;
import emporer.server.db.ConnectionFactory;
import emporer.server.db.ConnectionThreadLocal;
import emporer.server.user.UserAction;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.Socket;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

/**
 * SocketManager manages individual sockets concurrently as they make requests.
 */
public class SocketManager implements Runnable {
	private static final Logger LOGGER = Logger.getLogger(SocketManager.class.getName());

	private Socket socket;
	private Writer out;
	private Reader in;
	private BufferedInputStream inStream;
	private Integer sessionId = null;
	private Map<String, Integer> invalidLogins;
	private boolean loginSuccess = true;
	private User user;
	
	private UserAction userAction;

	/**
	 * Creates a SocketManager for the given socket.
	 * @param socket
	 */
	public SocketManager(final Socket socket) {
		this.socket = socket;
		userAction = new UserAction();
	}

	/**
	 * Runs the SocketManager thread.
	 */
	public void run() {

		try {
			//Retrieve the socket's streams
			out = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream(), "ASCII"));
			inStream = new BufferedInputStream(socket.getInputStream());
			in = new InputStreamReader(inStream);

			//Loop indefinitely while requests are still coming in
			for(;;) {
				//Read the request
				Request request = readRequest();

				//Client connection was closed
				if (request == null) {
					return;
				}

				//Process the Request
				Response response = processRequest(request);

				//Write the Response
				writeResponse(response);
			}
		} catch (final IOException ex) {
			LOGGER.warning(ex.getMessage());
		}
	}

	/**
	 * Processes a given Request based off of the Action of the request and
	 * then returns the appropriate Response.
	 * @param request
	 * @return Response */
	private Response processRequest(final Request request) {
		Response response = null;
		JSONObject responseJson = null;
		JSONObject requestJson = request.getRequest();
		Action action = request.getAction();
		
		Connection sessionConn;
		
		//Make sure that the request is for a login action if there is no session
		if (sessionId == null && !(Action.LOGIN.equals(action) || Action.REGISTER.equals(action))) {
			System.out.println("Not logged in");
			responseJson = new JSONObject();
			responseJson.put("msg", "User is not logged in!");
			response = new Response(false, responseJson);
			return response;
		}
		
		System.out.println("Thread: " + Thread.currentThread().getName());
		response = new Response();
		
		//Perform a different task depending on the Action
		switch (action) {
		case LOGIN:
			sessionConn = ConnectionFactory.generateNewConnection();
			ConnectionThreadLocal.set(sessionConn);
			user = userAction.validateLogin(requestJson.getString("username"), requestJson.getString("password"));
			if (user != null) {
				response.setSuccessful(true);
			    //Add the new session
				sessionId = SessionManager.addSession(user.getUserId());
				UserSessionThreadLocal.set(new UserSession(user.getUserId(), sessionId));
				SessionManager.addConnection(sessionId, sessionConn);
				System.out.println("User logged in: " + user.getNameLast() + ", " + user.getNameLast());
				response.setJSON(user.getJSON());

			} else {
				response.setSuccessful(false);
				responseJson = new JSONObject();
				responseJson.put("msg", "Incorrect login credentials.");
				response.setJSON(responseJson);
			}
			break;
		case REGISTER:
			sessionConn = ConnectionFactory.generateNewConnection();
			ConnectionThreadLocal.set(sessionConn);
			user = userAction.registerUser(requestJson.getString("email"), requestJson.getString("firstName"), 
					requestJson.getString("lastName"), requestJson.getString("password"));

			if (user != null) {
				response.setSuccessful(true);
				response.setJSON(user.getJSON());
			}
			try {
				ConnectionThreadLocal.get().close();
			} catch (SQLException e2) {
				// TODO Auto-generated catch block
				e2.printStackTrace();
			}
			ConnectionThreadLocal.set(null);
			break;
		default:
			break;
		}
		
		if (response != null && !response.isSuccessful() && response.getJSON() == null) {
			response.setSuccessful(false);
		}
		
		System.out.println("Finished processing request");
		return response;
	}

	/**
	 * Read an incoming request from the socket's input stream
	 * and return the populated Request object once reading from 
	 * the stream has been completed.
	 * @return Request 
	 * @throws IOException */
	private Request readRequest() throws IOException {
		//Read the request
		String requestStr = "";
		for(;;) {
			int c = in.read();
			if (c == '\r' || c == '\n')
                break;
			if (c == -1) {
				close();
				return null;
			}
			requestStr += (char) c;
		}

		//Create a new Request with what was read
		Request request = new Request(requestStr);
		LOGGER.info("Got Request: " + request.getJSON());

		return request;
	}

	/**
	 * Writes a given Response to the socket's ouput stream.
	 * @param response
	
	 * @throws IOException */
	private void writeResponse(final Response response) throws IOException {
		out.write(response.toString() + "\r");
		out.flush();
	}

	/**
	 * Performs the actions necessary to close the socket.
	 */
	private void close() {
		if (socket != null) {
			try {
				//Remove the open SESSION
				if (sessionId != null) {
					SessionManager.removeSession(sessionId);
				}

				//Close the socket
				socket.close();
				LOGGER.info("Socket closed");
			} catch (final IOException e) {
				LOGGER.warning("Could not close socket - Session ID: " + sessionId);
			}
		}
	}
	
}
